//*****************************************************************
// Questa libreria si occupa di gestire e realizzare le 
// funzioni per il trattamento e la trasformazione di un codice 
// sorgente rappresentato da un insieme di valori mnemonici 
// in un codice macchina, rappresentato da valori numerici.
// la trasformazione avviene trasformando un particolare tipo
// di codice sorgente che rappresenta la codifica in codici 
// mnemonici di programmi per il simulatore di Von Neumann 
// realizzato.
//
// Nome del progetto     : Compiler files
// Nome di questo file   : translater.cpp
// Librerie standards    : stdio.h
// Altri files associati : tplus.cpp tplus.h 
//                         vmachine.cpp vmachine.h
// 
// Creatore              : Giovanni Ceglia 
// Data ultima revisione : 22/05/2003
// Per contatti          : giovanniceglia@email.it 
//*****************************************************************

#include <stdio.h>
#include <iostream.h>
#include "tplus.h"
#include "vmachine.h"

// struttura che rappresenta e mantiene un vettore di stringhe
// opportunamente pre trattate.
struct mnemonic_code{
  string81 words[81];
  unsigned char count;
 }; 
 
 
// struttura astratta che serve a gestire l'oggetto programma.
struct programma{
   string81 labelname[50];
   unsigned char labelvalue[50];
   unsigned char labelcount;
   string81 labelmark[50];
   unsigned char markaddr[50];
   unsigned char markcount;
   unsigned char code[256];
   unsigned char PI;
 }; 

// sottoprogramma che si occupa di gestire il tutto.
int genera_programma(const string81, const string81);

// carica il codice sorgente in una struttura di tipo testo.
void caricacodice(const string81, testo&);

// effettua la precompilazione di una linea di testo di codice.
void parseline(mnemonic_code&, const string81);
 
// effettua una predecodifica del codice trasformando una linea
// di codice precedentemente trattata, in codice macchina, ma
// senza inserire gli indirizzi degli identificatori.
void decodeline(programma&, const mnemonic_code&);

// crea il file eseguibile dalla macchina simulata di Von Neumann
// e lo salva su disco.
void salvacodice(const string81, const programma& p);

// collega e realizza l'ultimo passaggio nella generazione del codice
// macchina, effettuando la trasformazioni degli indirizzi degli 
// identificatori.
void linker_prog(programma&);



void linker_prog(programma& p)
 {
  int i,j;
   for (i=0; i<p.markcount; i++) 
     {
      j=0;
      while ((j<p.labelcount) && (ceglia_strcmp(p.labelmark[i],p.labelname[j])!=true)) 
       {
        j++;
       } 
      if (ceglia_strcmp(p.labelmark[i],p.labelname[j])==true) {
          p.code[p.markaddr[i]]=p.labelvalue[j];
        }
     }  
 }

void salvacodice(const string81 sorgente, const programma& p)
 {
  FILE *f;
  f=fopen(sorgente,"wb");
  int i;
  for (i=0; i<p.PI; i++) putc(p.code[i],f);
  fclose(f);  
 }

// effettua una prima trasformazione del codice senza associare
// gli indirizzi degli identificatori.
void decodeline(programma& p, const mnemonic_code& c)
  {
   int i,j;
   if (c.count>0)   // viene controllato se nella riga ci sono istruzioni
     {
       int opcodes;
       if (c.words[0][0]=='@') {     // controlla se la riga contiene un identificatore
           //p.labelname[p.labelcount]=c.words[1];
           for (i=0; i<81; i++) p.labelname[p.labelcount][i]=c.words[1][i];
           p.labelvalue[p.labelcount]=p.PI;
           p.labelcount++;
         }
        else if (ceglia_strcmp(c.words[0],"BYTE")==true) { // verifica se c' una struttura di dati
          //p.labelname[p.labelcount]=c.words[1];
           for (i=0; i<81; i++) 
             p.labelname[p.labelcount][i]=c.words[1][i];p.labelvalue[p.labelcount]=p.PI;
           p.labelcount++;
           for (i=2; i<c.count; i++) {
              if (numero_dec(c.words[i])==true) {
                   p.code[p.PI]=strint(c.words[i]);
                   p.PI++;  
                 }
               else {
                   for (j=0; j<strsize(c.words[i]); j++) {
                      p.code[p.PI]=c.words[i][j];
                      p.PI++;
                    }  
                 }      
            }     
         }
       else { // se non  un identificatore e ne una struttura dati allora 
              // codifica l'istruzione che trova.
           p.code[p.PI]=istructioncode(c.words[0]);
           opcodes = (p.code[p.PI] / 64) + 1;
           if ((numero_dec(c.words[1])==true) && (opcodes >= 2)){
               p.code[p.PI+1]=strint(c.words[1]);
             }
            else if (opcodes >= 2) { // se gli operandi sono uno
               //p.labelmark[p.markcount]=c.words[1];
               for (i=0; i<81; i++) p.labelmark[p.markcount][i]=c.words[1][i];
               p.markaddr[p.markcount]=p.PI+1;
               p.markcount++;
               p.code[p.PI+1]=0;            
             }                       
           if ((numero_dec(c.words[2])==true) && (opcodes >= 3)) {
                p.code[p.PI+2]=strint(c.words[2]);
             }
            else  if (opcodes >= 3) { // se l'istruzione ha due operandi
                //p.labelmark[p.markcount]=c.words[2];
                for (i=0; i<81; i++) p.labelmark[p.markcount][i]=c.words[2][i];
                p.markaddr[p.markcount]=p.PI+2;
                p.markcount++;
                p.code[p.PI+2]=0;            
             }                       
           p.PI = p.PI + opcodes;
         } 
     }
  }


// effettua una prima trasformazione della riga di codice sorgente
// trasformandola in una pi facilmente gestibile.
void parseline(mnemonic_code& l, const string81 s)
 {
   int i=0,j=0;
   l.count=1;
   while (s[i]!=0){
      switch (s[i]) {
        case 32  : { 
                    if (j>0) {
                              l.words[l.count-1][j]=0;
                              j=0;
                              l.count++;
                             }   
                   } break; 
        case ',' : {
                     if (j>0) {
                               l.words[l.count-1][j]=0;
                               j=0;
                               l.count++;
                              }   
                   } break;
        case '"' : {
                     i++;
                     while (s[i]!='"') {
                       l.words[l.count-1][j]=s[i];
                       i++;
                       j++;
                      }
                     l.words[l.count-1][j]=0; 
                     j=0;
                     l.count++;   
                   } break;
        case '@' : {
                     l.words[l.count-1][j]='@';
                     l.words[l.count-1][j+1]=0;
                     j=0;
                     l.count++;
                   } break;
        default  : {
                     l.words[l.count-1][j]=s[i];
                     j++;
                   } break;
       }             
     i++;
    }
   l.words[l.count-1][j]=0; 
 }

// carica il file sorgente
void caricacodice(const string81 sorgente, testo& txt)
 {
  FILE *f;
  f=fopen(sorgente,"rt");
  txt.count=0;
  unsigned char i=0;
  char c;
  while (!feof(f)){
     c=getc(f);
     if (c==10) { 
                  if (i>0) {
                             txt.riga[txt.count][i]=0;
                             i=0;
                             txt.count++; 
                           }  
                } 
      else if (c>=32) {
                       txt.riga[txt.count][i]=c;
                       i++;
                     }  
   }
  fclose(f);  
 }

// gestisce il tutto generando il programma codificato.
int genera_programma(const string81 src, const string81 dst)
  {
    programma p;
    p.PI=0;
    p.labelcount=0;
    p.markcount=0;
    testo txt;
    mnemonic_code line;
    int r;
    caricacodice(src,txt);
    
    for (r=0; r < txt.count; r++)
     {  
      cout<<"  :: line("<<r<<"): "<<txt.riga[r]<<"\n";
      line.count=0;
      parseline(line,txt.riga[r]);
      decodeline(p,line);
     }
    linker_prog(p);
    salvacodice(dst,p); 
    return p.PI;
  }




