//*****************************************************************
// Questa libreria definisce ed implementa le funzioni utili
// alla gestione della struttura astratta che rappresenta la 
// macchina virtuale da simulare.
//
// Nome del progetto     : vmachine.h vmachine.cpp
// Creatore              : Giovanni Ceglia
// Data ultima revisione : 22/05/2003
//
// richiede iostrem.h, conio.h stdio.h
// inoltre necessita del modulo tplus.h e tplus.cpp
//*****************************************************************

#include <iostream.h>
#include <conio.h>
#include <stdio.h>
#include "tplus.h"

#if ! defined (OPCODESET)
#define OPCODESET

// definizione dei tipi di istruzioni e del nome e del codice 
// operativo associato ad esse.

// Istruzioni con nessun operando. 
#define RST 0    // Istruzione di Reset, interrompe la simulazione.
#define STD 1    // Setta a 1 il flag di Direzione.
#define STZ 2    // Setta a 1 il flag di Zero.
#define STO 3    // Setta a 1 il flag di Overflow. 
#define STP 4    // Setta a 1 il flag di Positivit
#define STN 5    // Setta a 1 il flag di Negativit
#define NOP 6    // Non esegue nessuna operazione.
#define CLD 7    // Cancella e mette a 0 il flag Direzione.
#define CLZ 8    // Cancella e mette a 0 il flag Zero.
#define CLO 9    // Cancella e mette a 0 il flag Overflow.
#define CLP 10   // Cancella e mette a 0 il flag Positivit
#define CLN 11   // Cancella e mette a 0 il flag Negativit

#define INCOFFST 12 // Incrementa il registro di Offset, il quale
                    //  un registro il cui valore viene sommato
                    // in qualsiasi operazione di accesso alla 
                    // memoria.
                    
#define DECOFFST 13 // Decrementa il registro di Offset.

// Istruzioni con 1 solo operando.
#define JMP 64   // Salto incondizionato a 8 bit. JMP address
#define JPD 65   // Salta se Direzione = 1        JPD address 
#define JPO 66   // Salta se Overflow = 1         JPO address
#define JPZ 67   // Salta se Zero = 1             JPZ address
#define JPP 68   // Salta se Positivo = 1         JPP address
#define JPN 69   // Salta se Negativo = 1         JPN address

#define INP 70   // Legge Stringa da Tastiera     INP address
#define WRT 71   // Scrive Stringa a Video        WRT address
#define INC 72   // Incrementa una cella di mem.  INC address
#define DEC 73   // Decrementa una cella di mem.  DEC address

#define VALOFFST 74 // Assegna un valore al registro di OffSet.

// Istruzioni con due operandi.

#define CMP 128  // Compara due celle di memoria CMP address1,address2
                 // se i valori sono uguali setta il flag ZERO = 1
                 // se address1<address2 setta il flag Negativo = 1
                 // se address1>address2 setta Positivo = 1 
#define ADD 129  // Somma         ADD addr1,addr2  --> addr1=addr1+addr2
#define SUB 130  // Sottrazione   SUB addr1,addr2  --> addr1=addr1-addr2
#define MUL 131  // Moltiplica    MUL addr1,addr2  --> addr1=addr1*addr2
#define DIV 132  // Divisione     DIV addr1,addr2  --> addr1=addr1/addr2
#define CPY 133  // Copia         CPY addr1,addr2  --> addr1=addr2
#define VAL 134  // Assegna Val.  VAL addr1,valore --> addr1=valore                


// definizione della struttura astratta  che rappresenta una macchina
// virtuale e che mantiene tutte le informazioni inerenti al suo avanzamento
// e funzionamento
struct virtual_machine {
  unsigned char virtual_flags;
  unsigned char virtual_memory[256];
  unsigned char virtual_PI;
  unsigned char reg_istruzione;
  unsigned char reg_opcode_A;
  unsigned char reg_opcode_B;
  unsigned char reg_offset;
 };
 
// definizione di un tipo che rappresenta il codice operativo 
// dell'istruzione.
typedef char lowcode;

// restituisce il codice operativo dell'istruzione, 
// conoscendone la rappresentazione come stringa.
extern lowcode istructioncode(const string81 s);

// carica un programma codificato in codice macchina, e lo 
// trasferisce nella memoria della macchina virtuale, ed inoltre
// inizializza la macchina per il suo avvio.
extern void carica_programma(const string81, virtual_machine&);

// funzione che  attivata quando la macchina ha il flag di debug
// attivo, e che permette di visualizzare lo stato della macchina.
extern void visualizza_stato_macchina(const virtual_machine&);

#endif
